<?php

namespace Drupal\jsonapi_flysystem_uploader\Plugin\QueueWorker;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Queue\QueueWorkerBase;
use Drupal\Core\Entity\EntityInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Site\Settings;

/**
 * Provides base functionality for the DataPush Queue Workers.
 */
abstract class DataPushBase extends QueueWorkerBase implements ContainerFactoryPluginInterface {

  /**
   * The entity storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $entityStorage;

  /**
   * Creates a new DataPushBase object.
   *
   * @param EntityStorageInterface $entity_storage
   *   The node storage.
   */
  public function __construct(EntityStorageInterface $entity_storage) {
    $this->entityStorage = $entity_storage;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $container
        ->get('entity.manager')
        // @todo Need to identify what entity is being used.
        ->getStorage('SOMETHING')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {
    /** @var EntityInterface $entity */
    $entity = $this->entityStorage->load($data->entity_id);
    return $this->pushFile($entity, $data->config);
  }

  /**
   * Publishes a node.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   * @param array $config
   *
   * @return int
   */
  protected function pushFile(EntityInterface $entity, array $config) {
    // For a given $entity object.
    // @todo Change this to use dependency injection.
    $output = \Drupal::service('jsonapi.entity.to_jsonapi')->serialize($entity);
    $entity_type_id = $entity->getEntityType()->id();
    $bundle_id = $entity->bundle();
    $uuid = $entity->uuid();
    $filename = $uuid . '.json';

    // Load the appropriate configuration.
    $config = $this->getEndpointConfig($entity);

    // The Flysystem endpoint to use.
    $endpoint = $config['flysystem'];

    // Work out the directory to store the file in.
    $directory = $bundle_id;
    if (!empty($config['directory'])) {
      $directory = trim($config['directory'], '/');
    }

    // Compile the full endpoint path.
    $path = $endpoint . '://' . $directory . '/' . $filename;

    // Make the path more system friendly.
    $path = file_stream_wrapper_uri_normalize($path);

    // Upload the file.
    file_put_contents($path, $output);

    // Close the connection.
    return 1;
  }

  /**
   * Obtain the config array for a specific entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity that needs its configuration obtained.
   * 
   * @return array
   *   One value must be present in the array:
   *   * flysystem - The name of the Flysystem configuration that is to be used.
   *     These must be defined in the normal Flysystem settings.
   *     @see $settings['flysystem']
   *   Two other values may be present:
   *   * upload - Whether or not this data structure should be uploaded.
   *     Defaults to FALSE.
   *   * directory - All files will be uploaded to the specified directory. If
   *     this is not available then the file will be stored in a directory with
   *     the same name as the bundle. Leading or trailing slashes are not needed
   *     and will be ignored.
   */
  protected function getEndpointConfig($entity) {
    // Load the settings for this object.
    $settings = Settings::get('jsonapi_flysystem_uploader');

    // Look for a config item for this bundle type.
    return $settings[$entity->getEntityType()->id()][$entity->bundle()];
  }

}
